;;; DO NOT MODIFY THIS FILE
(if (featurep 'footnote-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "footnote/_pkg.el")

(package-provide 'footnote :version 1.13 :type 'regular)

;;;***

;;;### (autoloads (footnote-mode Footnote-add-footnote) "footnote" "footnote/footnote.el")

(defcustom footnote-mode-line-string " FN" "*String to display in modes section of the mode-line." :group 'footnote)

(defvar footnote-prefix [(control ?c) ?\!] "\
*When not using message mode, the prefix to bind in `mode-specific-map'")

(autoload 'Footnote-add-footnote "footnote" "\
Add a numbered footnote.
The number the footnote receives is dependent upon the relative location
of any other previously existing footnotes.
If the variable `footnote-narrow-to-footnotes-when-editing' is set,
the buffer is narrowed to the footnote body.  The restriction is removed
by using `Footnote-back-to-message'." t nil)

(defvar footnote-mode-map nil "\
Keymap used for footnote minor mode.")

(unless footnote-mode-map (setq footnote-mode-map (make-sparse-keymap)) (define-key footnote-mode-map "a" 'Footnote-add-footnote) (define-key footnote-mode-map "b" 'Footnote-back-to-message) (define-key footnote-mode-map "c" 'Footnote-cycle-style) (define-key footnote-mode-map "d" 'Footnote-delete-footnote) (define-key footnote-mode-map "g" 'Footnote-goto-footnote) (define-key footnote-mode-map "r" 'Footnote-renumber-footnotes) (define-key footnote-mode-map "s" 'Footnote-set-style))

(defvar footnote-minor-mode-map nil "\
Keymap used for binding footnote minor mode.")

(unless footnote-minor-mode-map (define-key global-map footnote-prefix footnote-mode-map))

(autoload 'footnote-mode "footnote" "\
Toggle footnote minor mode.
\\<message-mode-map>
key		binding
---		-------

\\[Footnote-renumber-footnotes]		Footnote-renumber-footnotes
\\[Footnote-goto-footnote]		Footnote-goto-footnote
\\[Footnote-delete-footnote]		Footnote-delete-footnote
\\[Footnote-cycle-style]		Footnote-cycle-style
\\[Footnote-back-to-message]		Footnote-back-to-message
\\[Footnote-add-footnote]		Footnote-add-footnote
" t nil)

(when (fboundp 'add-minor-mode) (add-minor-mode 'footnote-mode footnote-mode-line-string footnote-minor-mode-map))

;;;***

(provide 'footnote-autoloads)
